/*
 * date-format.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles formatting dates.  Eventually should use whatever Moz functions
 * are available, for now... it doesn't.
 */

var earthMonths = [
    "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"
];

var earthMonthsAbbrv = new Array(earthMonths.length); // in case we get a new month, or - shut up.

var earthAM = "A";
var earthPM = "P";

var earth24HourClock = false;

var TimeTodayAt = "";
var TimeTomorrowAt = "";
var TimeTwoDaysAt = "";
var TimeIn = "";
var TimeHours = "";

var minSeparator = ":";
var secSeparator = ".";

function localizeTimeFormatStrings() {
    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var strings = strBundleService.createBundle("chrome://vanadielclock/locale/time.properties");
    for (var i in earthMonths) {
	earthMonthsAbbrv[i] = strings.GetStringFromName("month.abbrv." + earthMonths[i]);
	earthMonths[i] = strings.GetStringFromName("month." + earthMonths[i]);
    }
    TimeTodayAt = strings.GetStringFromName("clock.today.at");
    TimeTomorrowAt = strings.GetStringFromName("clock.tomorrow.at");
    TimeTwoDaysAt = strings.GetStringFromName("clock.twodays.at");
    // don't use this if set to "none"
    if (TimeTwoDaysAt == "none") {
	TimeTwoDaysAt = null;
    }
    TimeIn = strings.GetStringFromName("clock.in");
    TimeHours = strings.GetStringFromName("clock.hours");
    earthAM = strings.GetStringFromName("clock.am");
    earthPM = strings.GetStringFromName("clock.pm");

    // grab certain prefs
    var prefs = getVanadielClockPreferences();
    earth24HourClock = prefs.use24HourClock;
    minSeparator = prefs.minSeparator;
    secSeparator = prefs.secSeparator;
}

// Formats a delay in milliseconds - don't use for delays greater than
// a day.
function formatMilliDelay(millis) {
    // Always display "0.00" for negative delays - this probably means a delay is due
    // to be update but hasn't been, yet.
    if (millis < 0)
	return "0" + secSeparator + "00";
    // Round the delay off to nearest second.
    // This prevents weird "skips" in the delay caused by the timer not always being
    // updated on exactly 1 second.
    var sec = Math.floor(millis / 1000);
    var hour = Math.floor(sec / 3600);
    if (hour > 3) {
	return Math.floor((sec + 1800) / 3600) + " " + TimeHours;
    }
    var min = Math.floor(sec / 60);
    min -= hour * 60;
    var result = "";
    if (hour >= 1) {
	result = hour + minSeparator;
	if (min < 10)
	    result += "0";
    }
    sec -= hour * 3600 + min * 60;
    result += min + secSeparator;
    if (sec < 10)
	result += "0";
    return result + sec;
}

// Formats a delay between a given date and a given other date
function formatEarthDelay(firstDate, secondDate, showDelay, surpressToday) {
    var startDay = Math.floor((firstDate.getTime() - firstDate.getTimezoneOffset() * 60000) / 86400000);
    var eventDay = Math.floor((secondDate.getTime() - secondDate.getTimezoneOffset() * 60000) / 86400000);
//    dump("Showing delay between " + firstDate.getMonth() + "/" + firstDate.getDate() + " (" + startDay + ") and " + secondDate.getMonth() + "/" + secondDate.getDate() + " (" + eventDay + ")" + (showDelay ? " including" : " without") + " timer.\n");
//    dump("Time 1: " + firstDate.getTime() + " Time 2: " + secondDate.getTime());
    var delay = "";
    if (showDelay) {
	// If it's less than 6 hours, display the delay
	var wait = secondDate.getTime() - firstDate.getTime();
//	dump("Delay is currently " + wait + "ms\n");
	if (wait < (6 * 3600000)) {
	    delay = " (" + formatMilliDelay(wait) + ")";
//	    dump("Delay display:" + delay + "\n");
	}
    }
    if (startDay == eventDay) {
	// still today, just display the time
	var res = formatEarthTime(secondDate, false) + delay
	return surpressToday ? res : TimeTodayAt + " " + res;
    } else if (startDay + 1 == eventDay) {
	// Event is tomorrow
	return TimeTomorrowAt + " " + formatEarthTime(secondDate, false) + delay;
    } else if (TimeTwoDaysAt != null && startDay + 2 == eventDay) {
	// Event is two days from now
	return TimeTwoDaysAt + " " + formatEarthTime(secondDate, false) + delay;
    } else {
	return formatEarthDate(secondDate) + " " + formatEarthTime(secondDate, false);
    }
}

// Returns a string formatted in Earth time (based on the user's settings)
function formatEarthDate(time) {
    return time.getFullYear() + ", " +
	    earthMonths[time.getMonth()] + " " +
	    time.getDate();
}

// Returns a string formatted in Earth time (based on the user's settings)
function formatEarthTime(time, showSeconds) {
    var result = time.getHours();
    var am = false;
    if (!earth24HourClock) {
	if (result > 12) {
	    result -= 12;
	} else {
	    am = result < 12;
	    if (result == 0)
		result = "12";
	}
    }
    var t = time.getMinutes();
    result += minSeparator;
    if (t < 10)
	result += "0";
    result += t;
    if (showSeconds == null || showSeconds) {
	t = time.getSeconds();
	result += secSeparator;
	if (t < 10)
	    result += "0";
	result += t;
    }
    if (!earth24HourClock) {
	result += " " + (am ? earthAM : earthPM);
    }
    return result;
}
