/*
 * options.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the setting and retrieving of options.
 */

var prefs = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch);

var extPrefs;

function init() {
    extPrefs = getVanadielClockPreferences(prefs);
    document.getElementById("showClock").checked = extPrefs.showClock;
    var moon = document.getElementById("showMoon");
    moon.checked = extPrefs.showMoon;
    // only can show the moon when the clock is active
    moon.disabled = ! extPrefs.showClock;
    var date = document.getElementById("showDate");
    // only can show the date when the clock is active
    date.checked = extPrefs.showDate;
    date.disabled = ! extPrefs.showClock;
    document.getElementById("use24HourClock").checked = extPrefs.use24HourClock;
    document.getElementById("minSeparator").setAttribute("value", extPrefs.minSeparator);
    document.getElementById("secSeparator").setAttribute("value", extPrefs.secSeparator);
    document.getElementById("useCanvasMoon").checked = extPrefs.useMoonCanvas;
    document.getElementById("useIncrementalMoon").checked = extPrefs.useIncrementalMoon;
    document.getElementById("tintMoon").checked = extPrefs.useMoonTint;
    // try and find the version
    // we want to pluck out the "rv:1.8" part since that gives us the version
    // of the rendering engine (1.8 supports canvas, 1.7 doesn't)
    var verRegex = /rv:(\d+)\.(\d+)/;
    var m = verRegex.exec(navigator.userAgent);
    var maj = 0, min = 0;
    if (m != null) {
	maj = parseInt(m[1]);
	min = parseInt(m[2]);
    }
    if ((maj == 1 && min > 7) || maj > 1) {
	// canvas should be supported
	document.getElementById("canvasSupport").style.display = '-moz-groupbox';
    } else {
	extPrefs.useIncrementalMoon = false; // forcibly change this to false
    }
    // Center this dialog on the opener
    window.sizeToContent();
    var x = Math.floor((window.opener.outerWidth - window.outerWidth) / 2 + window.opener.screenX);
    var y = Math.floor((window.opener.outerHeight - window.outerHeight) / 2 + window.opener.screenY);
    window.moveTo(x, y);
}

function toggleClock() {
    var disabled = ! (document.getElementById("showClock").checked);
    document.getElementById("showMoon").disabled = disabled;
    document.getElementById("showDate").disabled = disabled;
}

function toggleMoonCanvas(checkbox) {
    var disabled = ! checkbox.checked;
    document.getElementById("useIncrementalMoon").disabled = disabled;
    document.getElementById("tintMoon").disabled = disabled;
}

function acceptChanges() {
    // called to commit current changes
    extPrefs.showClock = document.getElementById("showClock").checked;
    extPrefs.showMoon = document.getElementById("showMoon").checked;
    extPrefs.showDate = document.getElementById("showDate").checked;
    extPrefs.use24HourClock = document.getElementById("use24HourClock").checked;
    extPrefs.minSeparator = document.getElementById("minSeparator").value;
    extPrefs.secSeparator = document.getElementById("secSeparator").value;
    extPrefs.useMoonCanvas = document.getElementById("useCanvasMoon").checked;
    extPrefs.useIncrementalMoon = document.getElementById("useIncrementalMoon").checked;
    extPrefs.useMoonTint = document.getElementById("tintMoon").checked;
    extPrefs.save(prefs);
    Components.classes["@mozilla.org/observer-service;1"]
	    .getService(Components.interfaces.nsIObserverService)
	    .notifyObservers(null, "vanadielclock", "prefs");
    return true;
}
